@@ -19,11 +19,11 @@ The mapping is as follows:
 
 Th(A,x)                      -> A(x)
 Th((C1 ^ C2 ^ ... ^ Cn),x)   -> Th(C1,x) ^ Th(C2,x) ^ ... ^ Th(Cn,x) 
-Th((∀R.C),x)                 -> Th(C(y)) :- R(x,y)
+Th((‚àÄR.C),x)                 -> Th(C(y)) :- R(x,y)
 Tb(A(x))                     -> A(x)
 Tb((C1 ^ C2 ^ ... ^ Cn),x)   -> Tb(C1,x) ^ Tb(C2,x) ^ ... ^ Tb(Cn,x)
 Tb((C1 v C2 v ... v Cn),x)   -> Tb(C1,x) v Tb(C2,x) v ... v Tb(Cn,x)
-Tb((∃R.C),x)                ->  R(x,y) ^ Tb(C,y) 
+Tb((‚àÉR.C),x)                ->  R(x,y) ^ Tb(C,y) 
 
 In addition, basic logic tautologies are included in the DHL definition:
 
@@ -62,18 +62,20 @@ T(owl:TransitiveProperty(P))   -> P(x,z) :- P(x,y) ^ P(y,z)
 from __future__ import generators
 from sets import Set
 from rdflib import BNode, RDF, Namespace, Variable, RDFS
+from rdflib.util import first
 from rdflib.Collection import Collection
 from rdflib.store import Store,VALID_STORE, CORRUPTED_STORE, NO_STORE, UNKNOWN
-from rdflib.Literal import Literal
+from rdflib import Literal, URIRef
 from pprint import pprint, pformat
-import sys
-from rdflib.term_utils import *
+import sys, copy
+#from rdflib.term_utils import *
 from rdflib.Graph import QuotedGraph, Graph
 from rdflib.store.REGEXMatching import REGEXTerm, NATIVE_REGEX, PYTHON_REGEX
 from FuXi.Rete.RuleStore import Formula
 from FuXi.Rete.AlphaNode import AlphaNode
-from FuXi.Horn.PositiveConditions import And, Or, Uniterm, Condition, Atomic,SetOperator
-from FuXi.Horn.HornRules import Clause
+from FuXi.Horn.PositiveConditions import And, Or, Uniterm, Condition, Atomic,SetOperator,Exists
+from FuXi.Horn.HornRules import Clause,Rule
+from FuXi.Rete.Util import renderNetwork
 from cStringIO import StringIO
 
 non_DHL_OWL_Semantics=\
@@ -118,6 +120,7 @@ non_DHL_OWL_Semantics=\
 {?S owl:differentFrom ?O} => {?O owl:differentFrom ?S}.
 {?S owl:complementOf ?O} => {?O owl:complementOf ?S}.
 {?S owl:disjointWith ?O} => {?O owl:disjointWith ?S}.
+
 """
 
 OWL_NS    = Namespace("http://www.w3.org/2002/07/owl#")
@@ -146,15 +149,13 @@ def reduceAnd(left,right):
     
 def NormalizeClause(clause):
     def fetchFirst(gen):
-        gen=list(gen)
-        assert len(gen)==1
-        return gen[0]
+        return first(gen)
     if hasattr(clause.head,'next') and not isinstance(clause.head,Condition):
         clause.head = fetchFirst(clause.head)
     if hasattr(clause.body,'next') and not isinstance(clause.body,Condition):
         clause.body = fetchFirst(clause.body)
-    assert isinstance(clause.head,(Atomic,And,Clause)),repr(clause.head)
-    assert isinstance(clause.body,Condition),repr(clause.body)
+#    assert isinstance(clause.head,(Atomic,And,Clause)),repr(clause.head)
+#    assert isinstance(clause.body,Condition),repr(clause.body)
     if isinstance(clause.head,And):
         clause.head.formulae = reduce(reduceAnd,clause.head)
     if isinstance(clause.body,And):
@@ -166,7 +167,7 @@ class Clause:
     """
     The RETE-UL algorithm supports conjunctions of facts in the head of a rule
     i.e.:   H1 ^ H2 ^ ... ^ H3 :- B1 ^  ^ Bm
-    The Clause definition os overridden to permit this syntax (not allowed
+    The Clause definition is overridden to permit this syntax (not allowed
     in definite LP or Horn rules)
     
     In addition, since we allow (in definite Horn) entailments beyond simple facts
@@ -179,15 +180,37 @@ class Clause:
     def __repr__(self):
         return "%r :- %r"%(self.head,self.body)
 
-def MapDLPtoNetwork(network,factGraph):
-    for horn_clause in T(factGraph):
+    def n3(self):
+        return u'{ %s } => { %s }'%(self.body.n3(),self.head.n3())    
+
+def makeRule(clause,nsMap):
+    vars=set()
+    for child in clause.head:
+        if isinstance(child,Or):
+            #Disjunction in the head, skip this rule:
+            #When a disjunction occurs on the r.h.s. of a subclass axiom it 
+            #becomes a disjunction in the head of the corresponding rule, and 
+            #this cannot be handled within the def-Horn framework.            
+            return None
+        assert isinstance(child,Uniterm),repr(child)
+        vars.update([term for term in child.toRDFTuple() if isinstance(term,Variable)])
+    for child in clause.body:
+        assert isinstance(child,Uniterm),repr(child)
+        vars.update([term for term in child.toRDFTuple() if isinstance(term,Variable)])        
+    return Rule(clause,declare=vars,nsMapping=nsMap)
+
+def MapDLPtoNetwork(network,factGraph,complementExpansions=[]):
+    ruleset=[]
+    for horn_clause in T(factGraph,complementExpansions=complementExpansions):
 #        print "## RIF BLD Horn Rules: Before LloydTopor: ##\n",horn_clause
 #        print "## RIF BLD Horn Rules: After LloydTopor: ##"
         for tx_horn_clause in LloydToporTransformation(horn_clause):
+            tx_horn_clause = NormalizeClause(tx_horn_clause)
 #            print tx_horn_clause
             disj = [i for i in breadth_first(tx_horn_clause.body) if isinstance(i,Or)]
             import warnings
             if len(disj)>1:
+                raise
                 warnings.warn("No support for multiple disjunctions in the body:\n"+repr(tx_horn_clause),UserWarning,1)
             elif disj:
                 #Disjunctions in the body
@@ -195,16 +218,28 @@ def MapDLPtoNetwork(network,factGraph):
 #                print tx_horn_clause
                 disj = disj[0]
                 for item in disj:
+#                    print "\tDisjunction operand: ", item
                     #replace disj with item in tx_horn_clause.body
                     list(breadth_first_replace(tx_horn_clause.body,candidate=disj,replacement=item))
-                    ExtendN3Rules(network,NormalizeClause(tx_horn_clause))
-                    #restore
-                    for item in breadth_first_replace(tx_horn_clause.body,candidate=item,replacement=disj):
-                        pass
+                    #Then we want to clone the horn clause with the replacement
+                    tx_clause_clone = copy.deepcopy(tx_horn_clause)
+#                    print "\tClause after replacement: ", tx_clause_clone
+                    for hc in ExtendN3Rules(network,NormalizeClause(tx_clause_clone)):
+                        ruleset.append(makeRule(hc,network.nsMap))
+                    #restore the replaced term (for the subsequent iteration)
+                    list(breadth_first_replace(tx_horn_clause.body,candidate=item,replacement=disj))
             else:
-                #print tx_horn_clause
-                ExtendN3Rules(network,NormalizeClause(tx_horn_clause))            
+#                print "No Disjunction in the body"
+#                print tx_horn_clause
+                for hc in ExtendN3Rules(network,NormalizeClause(tx_horn_clause)):
+                    _rule=makeRule(hc,network.nsMap)
+                    if _rule is not None:
+                        ruleset.append(_rule)                    
+            #Extract free variables anre add rule to ruleset
 #        print "#######################"
+    print "########## Finished Building decision network from DLP ##########"
+    #renderNetwork(network).write_graphviz('out.dot')
+    return ruleset
 
 def IsaFactFormingConclusion(head):
     """
@@ -216,16 +251,19 @@ def IsaFactFormingConclusion(head):
     of the def-Horn ruleset are not restricted to be facts.' - Scan depth-first
     looking for Clauses
     """
-    if isinstance(head,SetOperator):
+    if isinstance(head,And):
         for i in head:
             if not IsaFactFormingConclusion(i):
                 return False
         return True
+    elif isinstance(head,Or):
+        return False
     elif isinstance(head,Atomic):
         return True
     elif isinstance(head,Clause):
         return False
     else:
+        print head
         raise
 
 def traverseClause(condition):
@@ -286,12 +324,13 @@ def ExtendN3Rules(network,horn_clause):
     """
     Extends the network with the given Horn clause (rule)
     """
+    rt=[]
     ruleStore = network.ruleStore
     lhs = BNode()
     rhs = BNode()
     assert isinstance(horn_clause.body,(And,Uniterm)),list(horn_clause.body)
     assert len(list(horn_clause.body))
-    #print horn_clause
+#    print horn_clause
     for term in horn_clause.body:
         ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(term.toRDFTuple())
     assert isinstance(horn_clause.head,(And,Uniterm))
@@ -300,19 +339,38 @@ def ExtendN3Rules(network,horn_clause):
 #                   for c in conjuncts])
     #print horn_clause
     if IsaFactFormingConclusion(horn_clause.head):
+        def extractBNodes(term):
+            if isinstance(term,BNode):
+                yield term
+            elif isinstance(term,Uniterm):
+                for t in term.toRDFTuple():
+                    if isinstance(t,BNode):
+                        yield t
+        exist=[list(extractBNodes(i)) for i in breadth_first(horn_clause.head)]
+        e=Exists(formula=horn_clause.head,
+                 declare=set(reduce(lambda x,y:x+y,exist,[])))        
+        if reduce(lambda x,y:x+y,exist):
+            horn_clause.head=e
+            assert e.declare,exist
         for term in horn_clause.head:
             assert not hasattr(term,'next')
-        ruleStore.formulae.setdefault(rhs,Formula(rhs)).append(term.toRDFTuple())
+            if isinstance(term,Or):
+                ruleStore.formulae.setdefault(rhs,Formula(rhs)).append(term)
+            else:
+                ruleStore.formulae.setdefault(rhs,Formula(rhs)).append(term.toRDFTuple())
         ruleStore.rules.append((ruleStore.formulae[lhs],ruleStore.formulae[rhs]))
         network.buildNetwork(iter(ruleStore.formulae[lhs]),
                              iter(ruleStore.formulae[rhs]),
-                             ruleStore.formulae[lhs],
-                             ruleStore.formulae[rhs])
+                             horn_clause)
         network.alphaNodes = [node for node in network.nodes.values() if isinstance(node,AlphaNode)]
+        rt.append(horn_clause)
     else:
         for hC in LloydToporTransformation(horn_clause,fullReduction=True):
+            rt.append(hC)
             #print "normalized clause: ", hC
-            ExtendN3Rules(network,hC)
+            for i in ExtendN3Rules(network,hC):
+                rt.append(hC)
+    return rt
 
 def generatorFlattener(gen):
     assert hasattr(gen,'next')
@@ -329,15 +387,20 @@ def generatorFlattener(gen):
     else:
         return i
 
-def T(owlGraph):
+def T(owlGraph,complementExpansions=[]):
     """
+    #Subsumption (purely for TBOX classification)
+    {?C rdfs:subClassOf ?SC. ?A rdfs:subClassOf ?C} => {?A rdfs:subClassOf ?SC}.
+    {?C owl:equivalentClass ?A} => {?C rdfs:subClassOf ?A. ?A rdfs:subClassOf ?C}.
+    {?C rdfs:subClassOf ?SC. ?SC rdfs:subClassOf ?C} => {?C owl:equivalentClass ?SC}.
+    
     T(rdfs:subClassOf(C,D))       -> Th(D(y)) :- Tb(C(y))
     
     T(owl:equivalentClass(C,D)) -> { T(rdfs:subClassOf(C,D) 
                                      T(rdfs:subClassOf(D,C) }
     
     A generator over the Logic Programming rules which correspond
-    to the DL subsumption axiom denoted via rdfs:subClassOf
+    to the DL  ( unary predicate logic ) subsumption axiom described via rdfs:subClassOf
     """
     for c,p,d in owlGraph.triples((None,RDFS.subClassOf,None)):
         yield NormalizeClause(Clause(Tb(owlGraph,c),Th(owlGraph,d)))
@@ -346,14 +409,36 @@ def T(owlGraph):
         yield NormalizeClause(Clause(Tb(owlGraph,c),Th(owlGraph,d)))
         yield NormalizeClause(Clause(Tb(owlGraph,d),Th(owlGraph,c)))
     for s,p,o in owlGraph.triples((None,OWL_NS.intersectionOf,None)):
-        if isinstance(s,URIRef):
-            #special case, owl:intersectionOf is a neccessary and sufficient
-            #criteria and should thus work in *both* directions
-            body = And([Uniterm(RDF.type,[Variable("X"),i],newNss=owlGraph.namespaces()) \
-                           for i in Collection(owlGraph,o)])
+        if s not in complementExpansions:
+            conjunction=[]
+            for bodyTerm in Collection(owlGraph,o):
+                bodyUniTerm = Uniterm(RDF.type,[Variable("X"),bodyTerm],
+                                      newNss=owlGraph.namespaces())                                    
+                classifyingClause = NormalizeClause(Clause(Tb(owlGraph,bodyTerm),
+                                                 bodyUniTerm))
+                if isinstance(bodyTerm,URIRef):
+                    conjunction.append(bodyUniTerm)
+                elif (bodyTerm,OWL_NS.someValuesFrom,None) in owlGraph or\
+                     (bodyTerm,OWL_NS.hasValue,None) in owlGraph:                    
+                    conjunction.extend(NormalizeClause(Clause(Tb(owlGraph,bodyTerm),None)).body)
+                elif (bodyTerm,OWL_NS.allValuesFrom,None) in owlGraph:
+                    conjunction.append(bodyUniTerm)                    
+                    yield classifyingClause
+                elif (bodyTerm,OWL_NS.hasValue,None) in owlGraph:
+                    conjunction.extend(NormalizeClause(Clause(Tb(owlGraph,bodyTerm),None)).body)
+                elif (bodyTerm,OWL_NS.unionOf,None) in owlGraph:
+                    conjunction.append(bodyUniTerm)                    
+                    yield classifyingClause
+            body = And(conjunction)
             head = Uniterm(RDF.type,[Variable("X"),s],newNss=owlGraph.namespaces())
+#            O1 ^ O2 ^ ... ^ On => S(?X)            
             yield Clause(body,head)
-            yield Clause(head,body)
+            if isinstance(s,URIRef):
+#                S(?X) => O1 ^ O2 ^ ... ^ On                
+    #            special case, owl:intersectionOf is a neccessary and sufficient
+    #            criteria and should thus work in *both* directions 
+    #            This rule is not added for anonymous classes
+                yield Clause(head,body)
         
     for s,p,o in owlGraph.triples((None,OWL_NS.unionOf,None)):
         if isinstance(s,URIRef):
@@ -403,6 +488,9 @@ def T(owlGraph):
             
 def LloydToporTransformation(clause,fullReduction=False):
     """
+    Tautological, common horn logic forms (useful for normalizing 
+    conjunctive & disjunctive clauses)
+    
     (H ^ H0) :- B                 -> { H  :- B
                                        H0 :- B }
     (H :- H0) :- B                -> H :- B ^ H0
@@ -416,8 +504,11 @@ def LloydToporTransformation(clause,fullReduction=False):
             yield Clause(atom,clause.head)
     elif isinstance(clause.head,Clause):
         yield Clause(And([clause.body,clause.head.body]),clause.head.head)
-    elif isinstance(clause.head,Or) or \
-         not isinstance(clause.body,Condition):
+    elif isinstance(clause.head,Or):
+        #Disjunction in the body, not supported by def-Horn
+        #skip
+        return
+    elif not isinstance(clause.body,Condition):
         print clause.head
         raise
     elif fullReduction and isinstance(clause.head,And):
@@ -436,7 +527,7 @@ def LloydToporTransformation(clause,fullReduction=False):
 def commonConjunctionMapping(owlGraph,conjuncts,innerFunc,variable=Variable("X")):
     """
     DHL: T*((C1 ^ C2 ^ ... ^ Cn),x)    -> T*(C1,x) ^ T*(C2,x) ^ ... ^ T*(Cn,x)
-    OWL: intersectionOf(c1 … cn) =>  EC(c1) ∩ … ∩ EC(cn)
+    OWL: intersectionOf(c1 ÔøΩ c2 ,..,cn) =>  EC(c1) ‚à© ‚Ä¶ ‚à© EC(cn)
     """
     conjuncts = Collection(owlGraph,conjuncts)
     return And([generatorFlattener(innerFunc(owlGraph,c,variable)) 
@@ -444,21 +535,16 @@ def commonConjunctionMapping(owlGraph,conjuncts,innerFunc,variable=Variable("X")
 
 def Th(owlGraph,_class,variable=Variable('X'),position=LHS):
     """
+    DLP head (antecedent) knowledge assertional forms (ABox assertions, conjunction of
+    ABox assertions, and universal role restriction assertions)
     Th(A,x)                      -> A(x)
     Th((C1 ^ C2 ^ ... ^ Cn),x)   -> Th(C1,x) ^ Th(C2,x) ^ ... ^ Th(Cn,x) 
-    Th((∀R.C),x)                -> Th(C(y)) :- R(x,y)
+    Th((‚àÄR.C),x)                -> Th(C(y)) :- R(x,y)
     """
     props = list(set(owlGraph.predicates(subject=_class)))
-    if OWL_NS.intersectionOf in props:
-        #http://www.w3.org/TR/owl-semantics/#owl_intersectionOf
-        for s,p,o in owlGraph.triples((_class,OWL_NS.intersectionOf,None)):
-            rt=commonConjunctionMapping(owlGraph,o,Th,variable=variable)
-            if isinstance(s,URIRef):
-                rt = rt.formulae.append(Uniterm(RDF.type,[variable,s],newNss=owlGraph.namespaces()))
-            yield rt
-    elif OWL_NS.allValuesFrom in props:
+    if OWL_NS.allValuesFrom in props:
         #http://www.w3.org/TR/owl-semantics/#owl_allValuesFrom
-        #restriction(p allValuesFrom(r))    {x ∈ O | <x,y> ∈ ER(p) implies y ∈ EC(r)}
+        #restriction(p allValuesFrom(r))    {x ‚àà O | <x,y> ‚àà ER(p) implies y ‚àà EC(r)}
         for s,p,o in owlGraph.triples((_class,OWL_NS.allValuesFrom,None)):
             prop = list(owlGraph.objects(subject=_class,predicate=OWL_NS.onProperty))[0]
             newVar = Variable(BNode())
@@ -467,7 +553,7 @@ def Th(owlGraph,_class,variable=Variable('X'),position=LHS):
                 yield Clause(body,head)
     elif OWL_NS.someValuesFrom in props:
         #http://www.w3.org/TR/owl-semantics/#someValuesFrom
-        #estriction(p someValuesFrom(e)) {x ∈ O | ∃ <x,y> ∈ ER(p) ∧ y ∈ EC(e)}
+        #estriction(p someValuesFrom(e)) {x ‚àà O | ‚àÉ <x,y> ‚àà ER(p) ‚àß y ‚àà EC(e)}
         for s,p,o in owlGraph.triples((_class,OWL_NS.someValuesFrom,None)):
             prop = list(owlGraph.objects(subject=_class,predicate=OWL_NS.onProperty))[0]
             newVar = BNode()
@@ -479,28 +565,24 @@ def Th(owlGraph,_class,variable=Variable('X'),position=LHS):
     
 def Tb(owlGraph,_class,variable=Variable('X')):
     """
+    DLP body (consequent knowledge assertional forms (ABox assertions, 
+    conjunction / disjunction of ABox assertions, and exisential role restriction assertions)
+    These are all common EL++ templates for KR
     Tb(A(x))                      -> A(x)
     Tb((C1 ^ C2 ^ ... ^ Cn),x)    -> Tb(C1,x) ^ Tb(C2,x) ^ ... ^ Tb(Cn,x)
     Tb((C1 v C2 v ... v Cn),x)    -> Tb(C1,x) v Tb(C2,x) v ... v Tb(Cn,x)
-    Tb((∃R.C),x)                 ->  R(x,y) ^ Tb(C,y) 
+    Tb((‚àÉR.C),x)                 ->  R(x,y) ^ Tb(C,y) 
     """
     props = list(set(owlGraph.predicates(subject=_class)))
-    if OWL_NS.intersectionOf in props:
-        #http://www.w3.org/TR/owl-semantics/#owl_intersectionOf
-        for s,p,o in owlGraph.triples((_class,OWL_NS.intersectionOf,None)):
-            rt=commonConjunctionMapping(owlGraph,o,Tb,variable=variable)
-            if isinstance(s,URIRef):
-                rt = rt.formulae.append(Uniterm(RDF.type,[variable,s],newNss=owlGraph.namespaces()))
-            yield rt
-    elif OWL_NS.unionOf in props:
+    if OWL_NS.unionOf in props and not isinstance(_class,URIRef):
         #http://www.w3.org/TR/owl-semantics/#owl_unionOf
-        #OWL semantics: unionOf(c1 … cn) => EC(c1) ∪ … ∪ EC(cn)
+        #OWL semantics: unionOf(c1 ‚Ä¶ cn) => EC(c1) ‚à™ ‚Ä¶ ‚à™ EC(cn)
         for s,p,o in owlGraph.triples((_class,OWL_NS.unionOf,None)):
             yield Or([Tb(owlGraph,c,variable=variable) \
                            for c in Collection(owlGraph,o)])
     elif OWL_NS.someValuesFrom in props:
-        #http://www.w3.org/TR/owl-semantics/#someValuesFrom
-        #estriction(p someValuesFrom(e)) {x ∈ O | ∃ <x,y> ∈ ER(p) ∧ y ∈ EC(e)}
+        #http://www.w3.org/TR/owl-semantics/#owl_someValuesFrom
+        #estriction(p someValuesFrom(e)) {x ‚àà O | ‚àÉ <x,y> ‚àà ER(p) ‚àß y ‚àà EC(e)}
         prop = list(owlGraph.objects(subject=_class,predicate=OWL_NS.onProperty))[0]
         o =list(owlGraph.objects(subject=_class,predicate=OWL_NS.someValuesFrom))[0]
         newVar = Variable(BNode())
@@ -508,6 +590,14 @@ def Tb(owlGraph,_class,variable=Variable('X')):
         head = Th(owlGraph,o,variable=newVar)
         yield And([Uniterm(prop,[variable,newVar],newNss=owlGraph.namespaces()),
                     generatorFlattener(Tb(owlGraph,o,variable=newVar))])
+    elif OWL_NS.hasValue in props:
+        #http://www.w3.org/TR/owl-semantics/#owl_hasValue
+        #Domain-specific rules for hasValue
+        #Can be achieved via pD semantics        
+        prop = list(owlGraph.objects(subject=_class,predicate=OWL_NS.onProperty))[0]
+        o =first(owlGraph.objects(subject=_class,predicate=OWL_NS.hasValue))
+        yield Uniterm(prop,[variable,o],newNss=owlGraph.namespaces())
     else:
         #simple class
+        #"Named" Uniterm
         yield Uniterm(RDF.type,[variable,_class],newNss=owlGraph.namespaces())
\ No newline at end of file
