@@ -1,5 +1,8 @@
 """
 A Rete Network Building and 'Evaluation' Implementation for RDFLib Graphs of Notation 3 rules.
+The DLP implementation uses this network to automatically building RETE decision trees for OWL forms of
+DLP
+
 Uses Python hashing mechanism to maximize the efficiency of the built pattern network.
 
 The network :
@@ -7,19 +10,24 @@ The network :
     - takes a fact (or the removal of a fact, perhaps?) and propagates down, starting from it's alpha nodes
     - stores inferred triples in provided triple source (an RDFLib graph) or a temporary IOMemory Graph by default
 
-This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 License.
-To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/2.5/ 
-or send a letter to Creative Commons, 543 Howard Street, 5th Floor, San Francisco, California, 94105, USA.    
 """
-import time
+from itertools import izip,ifilter
+import time,sys
 from sets import Set
 from pprint import pprint
+from cStringIO import StringIO
 from Util import xcombine
 from BetaNode import BetaNode, LEFT_MEMORY, RIGHT_MEMORY, PartialInstanciation
 from AlphaNode import AlphaNode, ReteToken, SUBJECT, PREDICATE, OBJECT, BuiltInAlphaNode
+from FuXi.Horn.HornRules import Clause, Ruleset
+from FuXi.Horn import ComplementExpansion
+from FuXi.Syntax.InfixOWL import *
+from FuXi.Horn.PositiveConditions import Uniterm, SetOperator
+from FuXi.DLP import MapDLPtoNetwork,non_DHL_OWL_Semantics
 #from FuXi.Rete.RuleStore import N3Builtin
 from Util import generateTokenSet,renderNetwork
 from rdflib import Variable, BNode, URIRef, Literal, Namespace,RDF,RDFS
+from rdflib.util import first
 from rdflib.Collection import Collection
 from rdflib.Graph import ConjunctiveGraph,QuotedGraph,ReadOnlyGraphAggregate, Graph
 from rdflib.syntax.NamespaceManager import NamespaceManager
@@ -29,6 +37,16 @@ OWL_NS    = Namespace("http://www.w3.org/2002/07/owl#")
 Any = None
 LOG = Namespace("http://www.w3.org/2000/10/swap/log#")
 
+#From itertools recipes
+def iteritems(mapping): 
+    return izip(mapping.iterkeys(),mapping.itervalues())
+
+def any(seq,pred=None):
+    """Returns True if pred(x) is true for at least one element in the iterable"""
+    for elem in ifilter(pred,seq):
+        return True
+    return False
+
 class HashablePatternList(object):
     """
     A hashable list of N3 statements which are patterns of a rule.  Order is disregarded
@@ -101,38 +119,50 @@ def _mulPatternWithSubstitutions(tokens,consequent,termNode):
     >>> token2 = token2.bindVariables(aNode)
     >>> inst = PartialInstanciation([token1,token2])
     >>> list(_mulPatternWithSubstitutions(inst,[Variable('O'),Variable('P'),Variable('S')]))
-    [(u'urn:uuid:alpha', u'http://www.w3.org/2002/07/owl#differentFrom', u'urn:uuid:beta'), (u'urn:uuid:beta', u'http://www.w3.org/2002/07/owl#differentFrom', u'urn:uuid:alpha')]
+    [(u'urn:uuid:alpha', 
+      u'http://www.w3.org/2002/07/owl#differentFrom', 
+      u'urn:uuid:beta'), 
+     (u'urn:uuid:beta', 
+      u'http://www.w3.org/2002/07/owl#differentFrom', 
+      u'urn:uuid:alpha')]
     """
     success = False
     for binding in tokens.bindings:        
         tripleVals = []
-        mismatchedTerms = [term for term in consequent if isinstance(term,Variable) and term not in binding]         
-        if not mismatchedTerms:
-            for term in consequent:
-                if isinstance(term,Variable):
-                    #try:                
-                    tripleVals.append(binding[term])
-                    #except:
-                    #    pass
-                else:                    
-                    tripleVals.append(term)
-            success = True
-            yield tuple(tripleVals)
-        else:
-            return
+        # if any(consequent,
+        #        lambda term:isinstance(term,Variable) and term not in binding):#  not mismatchedTerms:
+        #     return
+        # else:
+        for term in consequent:
+            if isinstance(term,Variable) and term in binding:
+                #try:                
+                tripleVals.append(binding[term])
+                #except:
+                #    pass
+            else:                    
+                tripleVals.append(term)
+        yield tuple(tripleVals),binding
 
-class ReteNetwork:      
+class ReteNetwork:
     """
     The Rete network.  The constructor takes an N3 rule graph, an identifier (a BNode by default), an 
     initial Set of Rete tokens that serve as the 'working memory', and an rdflib Graph to 
     add inferred triples to - by forward-chaining via Rete evaluation algorithm), 
     """
-    def __init__(self,ruleStore,name = None,initialWorkingMemory = None, inferredTarget = None,nsMap = {},graphVizOutFile=None):
-        from BuiltinPredicates import FILTERS        
+    def __init__(self,ruleStore,name = None,
+                 initialWorkingMemory = None, 
+                 inferredTarget = None,
+                 nsMap = {},
+                 graphVizOutFile=None,
+                 dontFinalize=False,
+                 goal=None):
+        from BuiltinPredicates import FILTERS
+        self.goal = goal        
         self.nsMap = nsMap
         self.name = name and name or BNode()
         self.nodes = {}
         self.alphaPatternHash = {}
+        self.ruleSet = set()
         for alphaPattern in xcombine(('1','0'),('1','0'),('1','0')):
             self.alphaPatternHash[tuple(alphaPattern)] = {}
         if inferredTarget is None:
@@ -140,33 +170,57 @@ class ReteNetwork:
         else:            
             self.inferredFacts = inferredTarget
         self.workingMemory = initialWorkingMemory and initialWorkingMemory or Set()
-        self.terminalNodes  = []
+        self.proofTracers = {}
+        self.terminalNodes  = set()
         self.instanciations = {}        
         start = time.time()
         self.ruleStore=ruleStore
-        self.ruleStore._finalize()
+        self.justifications = {}
+        self.dischargedBindings = {}
+        if not dontFinalize:
+            self.ruleStore._finalize()
         #self.ruleStore.optimizeRules()
         
         #'Universal truths' for a rule set are rules where the LHS is empty.  
         # Rather than automatically adding them to the working set, alpha nodes are 'notified'
         # of them, so they can be checked for while performing inter element tests.
         self.universalTruths = []
-        for lhs,rhs in self.ruleStore.rules:
-            if not len(lhs):
-                #If LHS is empty, then add to 'default' conflict set
-                self.universalTruths.extend(list(rhs))
-                continue
-            self.buildNetwork(iter(lhs),iter(rhs),lhs,rhs)
+        for rule in Ruleset(n3Rules=self.ruleStore.rules,nsMapping=self.nsMap):
+            self.buildNetwork(iter(rule.formula.body),
+                              iter(rule.formula.head),
+                              rule.formula)
         self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
+        self.alphaBuiltInNodes = [node for node in self.nodes.values() if isinstance(node,BuiltInAlphaNode)]
         self._setupDefaultRules()
-        print "Time to build production rule (RDFLib): %s seconds"%(time.time() - start)
+        print >>sys.stderr,"Time to build production rule (RDFLib): %s seconds"%(time.time() - start)
         if initialWorkingMemory:            
             start = time.time()          
             self.feedFactsToAdd(initialWorkingMemory)
-            print "Time to calculate closure on working memory: %s m seconds"%((time.time() - start) * 1000)            
+            print >>sys.stderr,"Time to calculate closure on working memory: %s m seconds"%((time.time() - start) * 1000)            
         if graphVizOutFile:
-            print "Writing out RETE network to ", graphVizOutFile
-            renderNetwork(self,nsMap=nsMap).write_graphviz(graphVizOutFile)
+            print >>sys.stderr,"Writing out RETE network to ", graphVizOutFile
+            renderNetwork(self,nsMap=nsMap).write(graphVizOutFile)
+                        
+    def setupDescriptionLogicProgramming(self,owlN3Graph,expanded):
+        noRules=len([rule 
+                    for rule in MapDLPtoNetwork(self,owlN3Graph,complementExpansions=expanded)])
+        self.parseN3Logic(StringIO(non_DHL_OWL_Semantics))
+        print "##### DLP rules setup",network
+        self.feedFactsToAdd(generateTokenSet(owlN3Graph))
+        print "##### DLP rules fired against OWL/RDF TBOX",network
+            
+    def parseN3Logic(self,src):
+        store=N3RuleStore(additionalBuiltins=self.ruleStore.filters)
+        Graph(store).parse(src,format='n3')
+        store._finalize()
+        assert len(store.rules),"There are no rules passed in!"
+        for rule in Ruleset(n3Rules=store.rules,
+                            nsMapping=self.nsMap):
+            self.buildNetwork(iter(rule.formula.body),
+                              iter(rule.formula.head),
+                              rule.formula)
+        self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
+        self.alphaBuiltInNodes = [node for node in self.nodes.values() if isinstance(node,BuiltInAlphaNode)]        
 
     def __repr__(self):
         total = 0 
@@ -175,10 +229,21 @@ class ReteNetwork:
                 total+=len(node.memories[LEFT_MEMORY])
                 total+=len(node.memories[RIGHT_MEMORY])
         
-        return "<Network: %s rules, %s nodes, %s tokens in working memory, %s inferred tokens>"%(len(self.ruleStore.rules),len(self.nodes),total,len(self.inferredFacts))
+        return "<Network: %s rules, %s nodes, %s tokens in working memory, %s inferred tokens>"%(len(self.terminalNodes),len(self.nodes),total,len(self.inferredFacts))
         
-    def closureGraph(self,sourceGraph):
-        return ReadOnlyGraphAggregate([sourceGraph,self.inferredFacts])
+    def closureGraph(self,sourceGraph,readOnly=True,store=None):
+        if readOnly:
+            roGraph=ReadOnlyGraphAggregate([sourceGraph,self.inferredFacts],
+                                           store=store)
+            for srcGraph in [sourceGraph,self.inferredFacts]:
+                for prefix,uri in srcGraph.namespaces():
+                    roGraph.namespace_manager.bind(prefix,uri)
+            return roGraph
+        else:
+            cg=ConjunctiveGraph()
+            cg+=sourceGraph
+            cg+=self.inferredFacts
+            return cg        
 
     def _setupDefaultRules(self):
         """
@@ -188,13 +253,15 @@ class ReteNetwork:
             if isinstance(node,AlphaNode):                
                 node.checkDefaultRule(self.universalTruths)
         
-    def reset(self):
+    def reset(self,newinferredFacts=None):
         "Reset the network by emptying the memory associated with all Beta Nodes nodes"
         for node in self.nodes.values():
             if isinstance(node,BetaNode):
                 node.memories[LEFT_MEMORY].reset()
                 node.memories[RIGHT_MEMORY].reset()
-        self.inferredFacts = Graph('IOMemory')
+        self.inferredFacts = newinferredFacts and newinferredFacts or Graph('IOMemory')
+        self.workingMemory = Set()
+        self._resetinstanciationStats()        
                                 
     def fireConsequent(self,tokens,termNode,debug=False):
         """
@@ -216,20 +283,29 @@ class ReteNetwork:
             print "%s from %s"%(tokens,termNode)
 
         newTokens = []
+        termNode.instanciatingTokens.add(tokens)
         for rhsTriple in termNode.consequent:
             if debug:
                 if not tokens.bindings:
                     tokens._generateBindings()
-            for inferredTriple in _mulPatternWithSubstitutions(tokens,rhsTriple,termNode):
-                #print inferredTriple, rhsTriple
-                if inferredTriple not in self.inferredFacts and ReteToken(inferredTriple) not in self.workingMemory:
-                    currIdx = self.instanciations.get(termNode,0)
-                    currIdx+=1
-                    self.instanciations[termNode] = currIdx
+            for inferredTriple,binding in _mulPatternWithSubstitutions(tokens,rhsTriple,termNode):
+                if [term for term in inferredTriple if isinstance(term,Variable)]:
+                    #Unfullfilled bindings
+                    continue
+                #print inferredTriple
+                inferredToken=ReteToken(inferredTriple)
+                self.proofTracers.setdefault(inferredTriple,[]).append(binding)
+                self.justifications.setdefault(inferredTriple,set()).add(termNode)
+                currIdx = self.instanciations.get(termNode,0)
+                currIdx+=1
+                self.instanciations[termNode] = currIdx
+                if inferredTriple not in self.inferredFacts and inferredToken not in self.workingMemory:                    
                     if debug:
                         print "Inferred triple: ", inferredTriple, " from ",termNode 
                     self.inferredFacts.add(inferredTriple)
-                    self.addWME(ReteToken(inferredTriple))
+                    self.addWME(inferredToken)
+#                    if self.goal is not None and self.goal in self.inferredFacts:
+#                        return                    
                 elif debug:
                     print "Inferred triple skipped: ", inferredTriple
     
@@ -249,7 +325,7 @@ class ReteNetwork:
         end        
         """
 #        print wme.asTuple()       
-        for termComb,termDict in self.alphaPatternHash.items():
+        for termComb,termDict in iteritems(self.alphaPatternHash):
             for alphaNode in termDict.get(wme.alphaNetworkHash(termComb),[]):
 #                print "\t## Activated AlphaNode ##"
 #                print "\t\t",termComb,wme.alphaNetworkHash(termComb)
@@ -291,9 +367,16 @@ class ReteNetwork:
         return node
     
     def _resetinstanciationStats(self):
-        self.instanciations = dict([(tNode,0) for tNode in self.terminalNodes])        
+        self.instanciations = dict([(tNode,0) for tNode in self.terminalNodes])
+        
+    def checkDuplicateRules(self):
+        checkedClauses={}
+        for tNode in self.terminalNodes:
+            collision = checkedClauses.get(tNode.clause)
+            assert collision is None,"%s collides with %s"%(tNode,checkedClauses[tNode.clause])
+            checkedClauses.setdefault(tNode.clause,[]).append(tNode) 
     
-    def buildNetwork(self,lhsIterator,rhsIterator,lhsFormula,rhsFormula):
+    def buildNetwork(self,lhsIterator,rhsIterator,clause):
         """
         Takes an iterator of triples in the LHS of an N3 rule and an iterator of the RHS and extends
         the Rete network, building / reusing Alpha 
@@ -307,11 +390,16 @@ class ReteNetwork:
             try:
                 currentPattern = lhsIterator.next()
                 #The LHS isn't done yet, stow away the current pattern
+                #We need to convert the Uniterm into a triple
+                if isinstance(currentPattern,Uniterm):
+                    currentPattern = currentPattern.toRDFTuple()
                 LHS.append(currentPattern)
             except StopIteration:                
                 #The LHS is done, need to initiate second pass to recursively build join / beta
-                #nodes towards a terminal node
-                consequents = list(rhsIterator)
+                #nodes towards a terminal node                
+                
+                #We need to convert the Uniterm into a triple
+                consequents = [isinstance(fact,Uniterm) and fact.toRDFTuple() or fact for fact in rhsIterator]
                 if matchedPatterns and matchedPatterns in self.nodes:
                     attachedPatterns.append(matchedPatterns)
                 elif matchedPatterns:
@@ -324,26 +412,29 @@ class ReteNetwork:
                     else:
                         paddedLHSPattern = HashablePatternList([None])+attachedPatterns[0]                    
                         terminalNode = self.nodes.get(paddedLHSPattern,BetaNode(None,node,aPassThru=True))
-                        self.nodes[paddedLHSPattern] = terminalNode    
-                        node.connectToBetaNode(terminalNode,RIGHT_MEMORY) 
-
-                    terminalNode.rule = (LHS,consequents)
-                    terminalNode.consequent.update(consequents)
-                    terminalNode.network    = self
-                    terminalNode.ruleFormulae = [lhsFormula,rhsFormula]
-                    self.terminalNodes.append(terminalNode)
-                    
+                        if True:#clause not in self.ruleSet:
+                            self.nodes[paddedLHSPattern] = terminalNode    
+                            node.connectToBetaNode(terminalNode,RIGHT_MEMORY)                            
+                    if True:#clause not in self.ruleSet:
+                        #self.ruleSet.add(clause)                        
+                        terminalNode.rule = (LHS,consequents)
+                        terminalNode.consequent.update(consequents)
+                        terminalNode.network    = self
+                        terminalNode.clause = clause
+                        self.terminalNodes.add(terminalNode)                    
                 else:              
                     for aP in attachedPatterns:
                         assert isinstance(aP,HashablePatternList),repr(aP)                    
                     terminalNode = self.attachBetaNodes(iter(attachedPatterns))
-                    
-                terminalNode.rule = (LHS,consequents)
-                terminalNode.consequent.update(consequents)
-                terminalNode.network    = self
-                terminalNode.ruleFormulae = [lhsFormula,rhsFormula]
-                self.terminalNodes.append(terminalNode)
-                self._resetinstanciationStats()
+                    if True:#clause not in self.ruleSet:
+                        #self.ruleSet.add(clause)
+                        terminalNode.rule = (LHS,consequents)
+                        terminalNode.consequent.update(consequents)
+                        terminalNode.network    = self
+                        terminalNode.clause = clause
+                        self.terminalNodes.add(terminalNode)
+                        self._resetinstanciationStats()                        
+                #self.checkDuplicateRules()
                 return
             if HashablePatternList([currentPattern]) in self.nodes:
                 #Current pattern matches an existing alpha node
@@ -422,6 +513,27 @@ class ReteNetwork:
         newBetaNode.connectIncomingNodes(firstNode,secondNode)
         return self.attachBetaNodes(patternIterator,newBNodePattern)
     
+def ComplementExpand(tBoxGraph,complementAnnotation):
+    complementExpanded=[]
+    for negativeClass in tBoxGraph.subjects(predicate=OWL_NS.complementOf):
+        containingList = first(tBoxGraph.subjects(RDF.first,negativeClass))
+        prevLink = None
+        while containingList:
+            prevLink = containingList
+            containingList = first(tBoxGraph.subjects(RDF.rest,containingList))
+        if prevLink:
+            for s,p,o in tBoxGraph.triples_choices((None,
+                                                [OWL_NS.intersectionOf,
+                                                 OWL_NS.unionOf],
+                                                 prevLink)):
+                if (s,complementAnnotation,None) in tBoxGraph:
+                    continue
+                _class = Class(s)
+                complementExpanded.append(s)
+                print "Added %s to complement expansion"%_class
+                ComplementExpansion(_class)
+    
+    
 def test():
     import doctest
     doctest.testmod()
