@@ -80,6 +80,8 @@ The '&' operator can be used to construct class intersection:
 >>> woman.identifier = exNs.Woman
 >>> woman
 ( ex:Female and ex:Human )
+>>> len(woman)
+2
 
 Enumerated classes can also be manipulated
 
@@ -106,6 +108,9 @@ import os
 from pprint import pprint
 from rdflib import Namespace
 from rdflib import plugin,RDF,RDFS,URIRef,BNode,Literal,Variable
+from rdflib.Literal import _XSD_NS
+from rdflib.Identifier import Identifier
+from rdflib.util import first
 from rdflib.store import Store
 from rdflib.Graph import Graph
 from rdflib.Collection import Collection
@@ -183,11 +188,35 @@ def manchesterSyntax(thing,store,boolean=None,transientList=False):
     assert thing is not None
     if boolean:
         if transientList:
+            liveChildren=iter(thing)
             children = [manchesterSyntax(child,store) for child in thing ]
         else:
+            liveChildren=iter(Collection(store,thing))
             children = [manchesterSyntax(child,store) for child in Collection(store,thing)]
         if boolean == OWL_NS.intersectionOf:
-            return '( '+ ' and '.join(children) + ' )'
+            childList=[]
+            named = []
+            for child in liveChildren:
+                if isinstance(child,URIRef):
+                    named.append(child)
+                else:
+                    childList.append(child)
+            if named:
+                def castToQName(x):
+                    prefix,uri,localName = store.compute_qname(x) 
+                    return u':'.join([prefix,localName])
+                
+                if len(named) > 1:
+                    prefix = '( '+ ' and '.join(map(castToQName,named)) + ' )'                
+                else:
+                    prefix = manchesterSyntax(named[0],store)
+                if childList:
+                    return prefix+ ' that '+' and '.join(
+                             map(lambda x:manchesterSyntax(x,store),childList))
+                else:
+                    return prefix
+            else:
+                return '( '+ ' and '.join(children) + ' )'
         elif boolean == OWL_NS.unionOf:
             return '( '+ ' or '.join(children) + ' )'
         elif boolean == OWL_NS.oneOf:
@@ -218,6 +247,9 @@ def manchesterSyntax(thing,store,boolean=None,transientList=False):
             prefix,uri,localName = store.compute_qname(thing) 
             qname = u':'.join([prefix,localName])
         except Exception,e:
+            if isinstance(thing,BNode):
+                return thing.n3()
+            return "<"+thing+">"
             print list(store.objects(subject=thing,predicate=RDF.type))
             raise
             return '[]'#+thing._id.encode('utf-8')+'</em>'            
@@ -231,18 +263,72 @@ def GetIdentifiedClasses(graph):
         if isinstance(c,URIRef):
             yield Class(c)
 
-class AnnotatibleTerms(object):
+class Individual(object):
+    """
+    A typed individual
+    """
+    factoryGraph = Graph()
+    def __init__(self, identifier=None,graph=None):
+        self.__identifier = identifier is not None and identifier or BNode()
+        if graph is None:
+            self.graph = self.factoryGraph
+        else:
+            self.graph = graph    
+        self.qname = None
+        if not isinstance(self.identifier,BNode):
+            try:
+                prefix,uri,localName = self.graph.compute_qname(self.identifier) 
+                self.qname = u':'.join([prefix,localName])
+            except:
+                pass
+    
+    def _get_type(self):
+        for _t in self.graph.objects(subject=self.identifier,predicate=RDF.type):
+            yield _t
+    def _set_type(self, kind):
+        if not kind:
+            return  
+        if isinstance(kind,(Individual,Identifier)):
+            self.graph.add((self.identifier,RDF.type,classOrIdentifier(kind)))
+        else:
+            for c in kind:
+                assert isinstance(c,(Individual,Identifier))
+                self.graph.add((self.identifier,RDF.type,classOrIdentifier(c)))
+    type = property(_get_type, _set_type)
+
+    def _get_identifier(self):
+        return self.__identifier
+    def _set_identifier(self, i):
+        assert i
+        if i != self.__identifier:
+            oldStmtsOut = [(p,o) for s,p,o in self.graph.triples((self.__identifier,None,None))]
+            oldStmtsIn  = [(s,p) for s,p,o in self.graph.triples((None,None,self.__identifier))]
+            for p1,o1 in oldStmtsOut:                
+                self.graph.remove((self.__identifier,p1,o1))
+            for s1,p1 in oldStmtsIn:                
+                self.graph.remove((s1,p1,self.__identifier))
+            self.__identifier = i
+            self.graph.addN([(i,p1,o1,self.graph) for p1,o1 in oldStmtsOut])
+            self.graph.addN([(s1,p1,i,self.graph) for s1,p1 in oldStmtsIn])
+    identifier = property(_get_identifier, _set_identifier)
+
+class AnnotatibleTerms(Individual):
     """
     Terms in an OWL ontology with rdfs:label and rdfs:comment
     """
+    def __init__(self,identifier,graph):
+        super(AnnotatibleTerms, self).__init__(identifier,graph)
     def _get_comment(self):
         for comment in self.graph.objects(subject=self.identifier,predicate=RDFS.comment):
-            yield comments
+            yield comment
     def _set_comment(self, comment):
         if not comment:
             return        
-        for c in comment:
-            self.graph.add((self.identifier,RDFS.comment,c))
+        if isinstance(comment,Identifier):
+            self.graph.add((self.identifier,RDFS.comment,comment))
+        else:
+            for c in comment:
+                self.graph.add((self.identifier,RDFS.comment,c))
     comment = property(_get_comment, _set_comment)
 
     def _get_seeAlso(self):
@@ -261,38 +347,22 @@ class AnnotatibleTerms(object):
     def _set_label(self, label):
         if not label:
             return        
-        for l in label:
-            self.graph.add((self.identifier,RDFS.label,l))
+        if isinstance(label,Identifier):
+            self.graph.add((self.identifier,RDFS.label,label))
+        else:
+            for l in label:
+                self.graph.add((self.identifier,RDFS.label,l))
     label = property(_get_label, _set_label)
 
 class Ontology(AnnotatibleTerms):
     """ The owl ontology metadata"""
-    factoryGraph = Graph()
-    def __init__(self, identifier=BNode(),imports=None,comment=None,
-                 graph=None):
-        self.__identifier = identifier
+    def __init__(self, identifier=None,imports=None,comment=None,graph=None):
+        super(Ontology, self).__init__(identifier,graph)
         self.imports = imports and imports or []
         self.comment = imports and imports or []
-        self.graph = graph or self.factoryGraph
         if (self.identifier,RDF.type,OWL_NS.Ontology) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL_NS.Ontology))
 
-    def _get_identifier(self):
-        return self.__identifier
-    def _set_identifier(self, i):
-        assert i
-        if i != self.__identifier:
-            oldStmtsOut = [(p,o) for s,p,o in self.graph.triples((self.__identifier,None,None))]
-            oldStmtsIn  = [(s,p) for s,p,o in self.graph.triples((None,None,self.__identifier))]
-            for p1,o1 in oldStmtsOut:                
-                self.graph.remove((self.__identifier,p1,o1))
-            for s1,p1 in oldStmtsIn:                
-                self.graph.remove((s1,p1,self.__identifier))
-            self.__identifier = i
-            self.graph.addN([(i,p1,o1,self.graph) for p1,o1 in oldStmtsOut])
-            self.graph.addN([(s1,p1,i,self.graph) for s1,p1 in oldStmtsIn])
-    identifier = property(_get_identifier, _set_identifier)
-
     def setVersion(self,version):
         self.graph.set((self.identifier,OWL_NS.versionInfo,version))
 
@@ -312,11 +382,31 @@ def AllClasses(graph):
         if c not in prevClasses:
             prevClasses.add(c)
             yield Class(c)            
+
+def AllProperties(graph):
+    prevProps=set()
+    for s,p,o in graph.triples_choices(
+               (None,RDF.type,[OWL_NS.Symmetric,
+                               OWL_NS.FunctionalProperty,
+                               OWL_NS.InverseFunctionalProperty,
+                               OWL_NS.TransitiveProperty,
+                               OWL_NS.DatatypeProperty,
+                               OWL_NS.ObjectProperty])):
+        if o in [OWL_NS.Symmetric,
+                 OWL_NS.InverseFunctionalProperty,
+                 OWL_NS.TransitiveProperty,
+                 OWL_NS.ObjectProperty]:
+            bType=OWL_NS.ObjectProperty
+        else:
+            bType=OWL_NS.DatatypeProperty
+        if s not in prevProps:
+            prevProps.add(s)
+            yield Property(s,
+                           graph=graph,
+                           baseType=bType)            
     
-def CastClass(c,graph):
-#    for kind in graph.triples_choices((classOrIdentifier(c),
-#                                       Restriction.restrictionKinds,
-#                                       None)):         
+def CastClass(c,graph=None):
+    graph = graph is None and c.factoryGraph or graph
     for kind in graph.objects(subject=classOrIdentifier(c),
                               predicate=RDF.type):
         if kind == OWL_NS.Restriction:
@@ -364,17 +454,10 @@ class Class(AnnotatibleTerms):
       description."
       
     """
-    factoryGraph = Graph()
-    
-    def __init__(self, identifier=BNode(),subClassOf=None,equivalentClass=None,
-                       disjointWith=None,complementOf=None,graph=None,skipOWLClassMembership = False,
-                       comment=None):
-        self.__identifier = identifier
-        self.qname = None
-        self.graph = graph is not None and graph or self.factoryGraph
-        if not isinstance(identifier,BNode):
-            prefix,uri,localName = self.graph.compute_qname(identifier) 
-            self.qname = u':'.join([prefix,localName])
+    def __init__(self, identifier=None,subClassOf=None,equivalentClass=None,
+                       disjointWith=None,complementOf=None,graph=None,
+                       skipOWLClassMembership = False,comment=None):
+        super(Class, self).__init__(identifier,graph)
         if not skipOWLClassMembership and (self.identifier,RDF.type,OWL_NS.Class) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL_NS.Class))
         
@@ -384,27 +467,6 @@ class Class(AnnotatibleTerms):
         if complementOf:
             self.complementOf    = complementOf
         self.comment = comment and comment or []
-
-#    def _get_label(self):
-#        return self.__identifier
-#    def _set_label(self, i):
-#        assert i
-
-    def _get_identifier(self):
-        return self.__identifier
-    def _set_identifier(self, i):
-        assert i
-        if i != self.__identifier:
-            oldStmtsOut = [(p,o) for s,p,o in self.graph.triples((self.__identifier,None,None))]
-            oldStmtsIn  = [(s,p) for s,p,o in self.graph.triples((None,None,self.__identifier))]
-            for p1,o1 in oldStmtsOut:                
-                self.graph.remove((self.__identifier,p1,o1))
-            for s1,p1 in oldStmtsIn:                
-                self.graph.remove((s1,p1,self.__identifier))
-            self.__identifier = i
-            self.graph.addN([(i,p1,o1,self.graph) for p1,o1 in oldStmtsOut])
-            self.graph.addN([(s1,p1,i,self.graph) for s1,p1 in oldStmtsIn])
-    identifier = property(_get_identifier, _set_identifier)
     
     def __iadd__(self, other):
         assert isinstance(other,Class)
@@ -415,6 +477,12 @@ class Class(AnnotatibleTerms):
         assert isinstance(other,Class)
         self.graph.remove((classOrIdentifier(other),RDFS.subClassOf,self.identifier))
         return self
+    
+    def __invert__(self):
+        """
+        Shorthand for Manchester syntax's not operator
+        """
+        return Class(complementOf=self)
 
     def __or__(self,other):
         """
@@ -486,9 +554,6 @@ class Class(AnnotatibleTerms):
             self.graph.add((self.identifier,RDFS.seeAlso,link))
     seeAlso = property(_get_seeAlso, _set_seeAlso)
     
-#    def __str__(self):
-#        return str(self.identifier)
-
     def isPrimitive(self):
         if (self.identifier,RDF.type,OWL_NS.Restriction) in self.graph:
             return False
@@ -533,9 +598,10 @@ class Class(AnnotatibleTerms):
                 scJoin = ', '
             necStatements = [
               isinstance(s,Class) and isinstance(self.identifier,BNode) and
-                                      repr(BooleanClass(classOrIdentifier(s),
-                                                        operator=None,
-                                                        graph=self.graph)) or 
+                                      repr(CastClass(s,self.graph)) or
+                                      #repr(BooleanClass(classOrIdentifier(s),
+                                      #                  operator=None,
+                                      #                  graph=self.graph)) or 
               manchesterSyntax(classOrIdentifier(s),self.graph) for s in sc]
             if necStatements:
                 klassKind = "Primitive Type %s"%label
@@ -564,7 +630,7 @@ class Class(AnnotatibleTerms):
             (descr and "\n    %s"%descr[0] or '') + ' . '.join(exprs) or ' . '.join(exprs)
         else:
             klassDescr = full and (descr and "\n    %s"%descr[0] or '') or '' + ' . '.join(exprs)
-        return "Class: %s "%(isinstance(self.identifier,BNode) and '[]' or self.qname)+klassDescr
+        return (isinstance(self.identifier,BNode) and "Some Class " or "Class: %s "%self.qname)+klassDescr
 
 class OWLRDFListProxy(object):
     def __init__(self,rdfList,members=None,graph=Graph()):
@@ -593,7 +659,10 @@ class OWLRDFListProxy(object):
         self._rdfList[key] = classOrIdentifier(value)
         
     def __delitem__(self, key):
-        del self._rdfList[key]        
+        del self._rdfList[key]    
+        
+    def clear(self):
+        self._rdfList.clear()    
 
     def __iter__(self):
         for item in self._rdfList:
@@ -619,7 +688,7 @@ class EnumeratedClass(Class,OWLRDFListProxy):
     _operator = OWL_NS.oneOf
     def isPrimitive(self):
         return False
-    def __init__(self, identifier=BNode(),members=None,graph=Graph()):
+    def __init__(self, identifier=None,members=None,graph=None):
         Class.__init__(self,identifier,graph = graph)
         members = members and members or []
         rdfList = list(self.graph.objects(predicate=OWL_NS.oneOf,subject=self.identifier))
@@ -639,27 +708,57 @@ class BooleanClass(Class,OWLRDFListProxy):
     owl:complementOf is an attribute of Class, however
     
     """
-    def __init__(self,identifier=BNode(),operator=OWL_NS.intersectionOf,
-                 members=None,graph=Graph()):
+    def __init__(self,identifier=None,operator=OWL_NS.intersectionOf,
+                 members=None,graph=None):
         if operator is None:
-            iters=0
+            props=[]
             for s,p,o in graph.triples_choices((identifier,
                                                 [OWL_NS.intersectionOf,
                                                  OWL_NS.unionOf],
                                                  None)):
-                iters+=1
+                props.append(p)
                 operator = p
-            assert len(iters)==1
+            assert len(props)==1,repr(props)
         Class.__init__(self,identifier,graph = graph)
-        members = members and members or []
         assert operator in [OWL_NS.intersectionOf,OWL_NS.unionOf], str(operator)
         self._operator = operator
         rdfList = list(self.graph.objects(predicate=operator,subject=self.identifier))
+        assert not members or not rdfList,"This is a previous boolean class description!"+repr(Collection(self.graph,rdfList[0]).n3())        
         OWLRDFListProxy.__init__(self, rdfList, members, graph = graph)
 
     def isPrimitive(self):
         return False
 
+    def changeOperator(self,newOperator):
+        """
+        Converts a unionOf / intersectionOf class expression into one 
+        that instead uses the given operator
+        
+        
+        >>> testGraph = Graph()
+        >>> Individual.factoryGraph = testGraph
+        >>> EX = Namespace("http://example.com/")
+        >>> namespace_manager = NamespaceManager(Graph())
+        >>> namespace_manager.bind('ex', EX, override=False)
+        >>> testGraph.namespace_manager = namespace_manager
+        >>> fire  = Class(EX.Fire)
+        >>> water = Class(EX.Water) 
+        >>> testClass = BooleanClass(members=[fire,water])
+        >>> testClass
+        ( ex:Fire and ex:Water )
+        >>> testClass.changeOperator(OWL_NS.unionOf)
+        >>> testClass
+        ( ex:Fire or ex:Water )
+        >>> try: testClass.changeOperator(OWL_NS.unionOf)
+        ... except Exception, e: print e
+        The new operator is already being used!
+        
+        """
+        assert newOperator != self._operator,"The new operator is already being used!"
+        self.graph.remove((self.identifier,self._operator,self._rdfList.uri))
+        self.graph.add((self.identifier,newOperator,self._rdfList.uri))
+        self._operator = newOperator
+
     def __repr__(self):
         """
         Returns the Manchester Syntax equivalent for this class
@@ -698,7 +797,7 @@ class Restriction(Class):
     
     def __init__(self,onProperty,graph = Graph(),allValuesFrom=None,someValuesFrom=None,value=None,
                       cardinality=None,maxCardinality=None,minCardinality=None,identifier=None):
-        super(Restriction, self).__init__(identifier and identifier or BNode(),
+        super(Restriction, self).__init__(identifier,
                                           graph=graph,
                                           skipOWLClassMembership=True)
         if (self.identifier,OWL_NS.onProperty,propertyOrIdentifier(onProperty)) not in graph:
@@ -735,7 +834,7 @@ class Restriction(Class):
 
     def _get_allValuesFrom(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.allValuesFrom):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_allValuesFrom(self, other):
         triple = (self.identifier,OWL_NS.allValuesFrom,classOrIdentifier(other))
@@ -749,7 +848,7 @@ class Restriction(Class):
 
     def _get_someValuesFrom(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.someValuesFrom):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_someValuesFrom(self, other):
         triple = (self.identifier,OWL_NS.someValuesFrom,classOrIdentifier(other))
@@ -763,7 +862,7 @@ class Restriction(Class):
 
     def _get_hasValue(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.hasValue):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_hasValue(self, other):
         triple = (self.identifier,OWL_NS.hasValue,classOrIdentifier(other))
@@ -777,7 +876,7 @@ class Restriction(Class):
 
     def _get_cardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.cardinality):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_cardinality(self, other):
         triple = (self.identifier,OWL_NS.cardinality,classOrIdentifier(other))
@@ -791,7 +890,7 @@ class Restriction(Class):
 
     def _get_maxCardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.maxCardinality):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_maxCardinality(self, other):
         triple = (self.identifier,OWL_NS.maxCardinality,classOrIdentifier(other))
@@ -805,7 +904,7 @@ class Restriction(Class):
 
     def _get_minCardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.minCardinality):
-            return Class(dc,graph=self.graph)
+            return Class(i,graph=self.graph)
         return None
     def _set_minCardinality(self, other):
         triple = (self.identifier,OWL_NS.minCardinality,classOrIdentifier(other))
@@ -837,7 +936,14 @@ only     = Infix(lambda prop,_class: Restriction(prop,graph=_class.graph,allValu
 max      = Infix(lambda prop,_class: Restriction(prop,graph=prop.graph,maxCardinality=_class))
 min      = Infix(lambda prop,_class: Restriction(prop,graph=prop.graph,minCardinality=_class))
 exactly  = Infix(lambda prop,_class: Restriction(prop,graph=prop.graph,cardinality=_class))
-value    = Infix(lambda prop,_class: Restriction(prop,graph=prop.graph,hasValue=_class))
+value    = Infix(lambda prop,_class: Restriction(prop,graph=prop.graph,value=_class))
+
+PropertyAbstractSyntax=\
+"""
+%s( %s { %s } 
+%s
+{ 'super(' datavaluedPropertyID ')'} ['Functional']
+{ domain( %s ) } { range( %s ) } )"""
 
 class Property(AnnotatibleTerms):
     """
@@ -851,39 +957,64 @@ class Property(AnnotatibleTerms):
                   'Transitive' ]
                 { 'domain(' description ')' } { 'range(' description ')' } ')    
     """
-    factoryGraph = Graph()
-    def __init__(self,identifier=BNode(),graph = None,baseType=OWL_NS.ObjectProperty,
+    def __init__(self,identifier=None,graph = None,baseType=OWL_NS.ObjectProperty,
                       subPropertyOf=None,domain=None,range=None,inverseOf=None,
                       otherType=None,equivalentProperty=None,comment=None):
-        self.__identifier = identifier
-        self.graph = graph is not None and graph or self.factoryGraph        
-        assert not isinstance(identifier,BNode)
-        prefix,uri,localName = self.graph.compute_qname(identifier) 
-        self.qname = u':'.join([prefix,localName])
+        super(Property, self).__init__(identifier,graph)
+        assert not isinstance(self.identifier,BNode)
         if (self.identifier,RDF.type,baseType) not in self.graph:
             self.graph.add((self.identifier,RDF.type,baseType))
+        self._baseType=baseType
         self.subPropertyOf = subPropertyOf
         self.inverseOf     = inverseOf
         self.domain        = domain
         self.range         = range
         self.comment = comment and comment or []
-        
-    def _get_identifier(self):
-        return self.__identifier
-    def _set_identifier(self, i):
-        assert i
-        if i != self.__identifier:
-            oldStmtsOut = [(p,o) for s,p,o in self.graph.triples((self.__identifier,None,None))]
-            oldStmtsIn  = [(s,p) for s,p,o in self.graph.triples((None,None,self.__identifier))]
-            for p1,o1 in oldStmtsOut:                
-                self.graph.remove((self.__identifier,p1,o1))
-            for s1,p1 in oldStmtsIn:                
-                self.graph.remove((s1,p1,self.__identifier))
-            self.__identifier = i
-            self.graph.addN([(i,p1,o1,self.graph) for p1,o1 in oldStmtsOut])
-            self.graph.addN([(s1,p1,i,self.graph) for s1,p1 in oldStmtsIn])
-    identifier = property(_get_identifier, _set_identifier)
 
+    def __repr__(self):
+        rt=[]
+        if OWL_NS.ObjectProperty in self.type:
+            rt.append('ObjectProperty( %s annotation(%s)'\
+                       %(self.qname,first(self.comment) and first(self.comment) or ''))
+            if first(self.inverseOf):
+                rt.append("  inverseOf( %s )%s"%(first(self.inverseOf),
+                            OWL_NS.Symmetric in self.type and ' Symmetric' or ''))
+            for s,p,roleType in self.graph.triples_choices((self.identifier,
+                                                            RDF.type,
+                                                            [OWL_NS.Functional,
+                                                             OWL_NS.InverseFunctionalProperty,
+                                                             OWL_NS.Transitive])):
+                rt.append(roleType.split(OWL_NS)[-1])
+        else:
+            rt.append('DatatypeProperty( %s %s'\
+                       %(self.qname,first(self.comment) and first(self.comment) or ''))            
+            for s,p,roleType in self.graph.triples((self.identifier,
+                                                    RDF.type,
+                                                    OWL_NS.Functional)):
+                rt.append('   Functional')
+        def canonicalName(term,g):
+            normalizedName=classOrIdentifier(term)
+            if isinstance(normalizedName,BNode):
+                return term
+            elif normalizedName.startswith(_XSD_NS):
+                return term
+            elif first(g.triples_choices((
+                      normalizedName,
+                      [OWL_NS.unionOf,
+                       OWL_NS.intersectionOf],None))):
+                return repr(term)
+            else:
+                return str(term.qname)
+        rt.append(' '.join(["   super( %s )"%canonicalName(superP,self.graph) 
+                              for superP in self.subPropertyOf]))                        
+        rt.append(' '.join(["   domain( %s )"% canonicalName(domain,self.graph) 
+                              for domain in self.domain]))
+        rt.append(' '.join(["   range( %s )"%canonicalName(range,self.graph) 
+                              for range in self.range]))
+        rt='\n'.join([expr for expr in rt if expr])
+        rt+='\n)'
+        return rt
+                    
     def _get_subPropertyOf(self):
         for anc in self.graph.objects(subject=self.identifier,predicate=RDFS.subPropertyOf):
             yield Property(anc,graph=self.graph)
@@ -903,53 +1034,35 @@ class Property(AnnotatibleTerms):
         self.graph.add((self.identifier,OWL_NS.inverseOf,classOrIdentifier(other)))
     inverseOf = property(_get_inverseOf, _set_inverseOf)
 
-    def _get_comment(self):
-        for comment in self.graph.objects(subject=self.identifier,predicate=RDFS.comment):
-            yield comment
-    def _set_comment(self, comments):
-        if not comments:
-            return     
-        for comment in comments:   
-            self.graph.add((self.identifier,RDFS.comment,comment))
-    comment = property(_get_comment, _set_comment)
-
     def _get_domain(self):
         for dom in self.graph.objects(subject=self.identifier,predicate=RDFS.domain):
             yield Class(dom,graph=self.graph)
     def _set_domain(self, other):
         if not other:
-            return        
-        for dom in other:
-            self.graph.add((self.identifier,RDFS.domain,classOrIdentifier(dom)))
+            return
+        if isinstance(other,(Individual,Identifier)):
+            self.graph.add((self.identifier,RDFS.domain,classOrIdentifier(other)))
+        else:
+            for dom in other:
+                self.graph.add((self.identifier,RDFS.domain,classOrIdentifier(dom)))
     domain = property(_get_domain, _set_domain)
 
-
     def _get_range(self):
         for ran in self.graph.objects(subject=self.identifier,predicate=RDFS.range):
             yield Class(ran,graph=self.graph)
     def _set_range(self, ranges):
         if not ranges:
-            return        
-        for range in ranges:
-            self.graph.add((self.identifier,RDFS.range,classOrIdentifier(range)))
+            return
+        if isinstance(ranges,(Individual,Identifier)):
+            self.graph.add((self.identifier,RDFS.range,classOrIdentifier(ranges)))
+        else:        
+            for range in ranges:
+                self.graph.add((self.identifier,RDFS.range,classOrIdentifier(range)))
     range = property(_get_range, _set_range)
         
 def test():
     import doctest
     doctest.testmod()
-    galenGraph = Graph()
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/SDB-local/Base/owl/DataNodes.owl'))
-    galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/bfo-1.0.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/OBI.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/problem-oriented-medical-record.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/InformationObjects.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/ExtendedDnS.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/DOLCE-Lite.owl'))
-    #galenGraph.parse(os.path.join(os.path.dirname(__file__), '/home/chimezie/workspace/Ontologies/Plans.owl'))
-    graph=galenGraph
-    for c in graph.subjects(predicate=RDF.type,object=OWL_NS.Class):
-        if isinstance(c,URIRef):
-            print Class(c,graph=graph).__repr__(True),"\n"
 
 if __name__ == '__main__':
     test()
