@@ -1,9 +1,10 @@
-from pprint import pprint
+from pprint import pprint, pformat
 from sets import Set
 from FuXi.Rete import *
 from FuXi.Rete.AlphaNode import SUBJECT,PREDICATE,OBJECT,VARIABLE
 from FuXi.Rete.RuleStore import N3RuleStore
 from FuXi.Rete.Util import renderNetwork,generateTokenSet
+from FuXi.DLP import MapDLPtoNetwork, non_DHL_OWL_Semantics
 from rdflib.Namespace import Namespace
 from rdflib import plugin,RDF,RDFS,URIRef,URIRef
 from rdflib.store import Store
@@ -12,7 +13,6 @@ from rdflib.Graph import Graph,ReadOnlyGraphAggregate,ConjunctiveGraph
 from rdflib.syntax.NamespaceManager import NamespaceManager
 from glob import glob
 from rdflib.sparql.bison import Parse
-from rdflib.sparql.bison.SPARQLEvaluate import Evaluate
 import unittest, os, time
 
 RDFLIB_CONNECTION=''
@@ -75,7 +75,6 @@ Tests2Skip = [
     'OWL/oneOf/Manifest003.rdf', #logical set equivalence?  Probably needs a built-in to support
     'OWL/differentFrom/Manifest002.rdf'  ,#needs log:notEqualTo
     'OWL/distinctMembers/Manifest001.rdf',#needs log:notEqualTo
-    'OWL/intersectionOf/Manifest001.rdf' ,#can't calculate identical class extensions
     'OWL/unionOf/Manifest002.rdf',#can't implement set theoretic union for owl:unionOf.
     'OWL/InverseFunctionalProperty/Manifest001.rdf',#owl:sameIndividualAs deprecated
     'OWL/FunctionalProperty/Manifest001.rdf', #owl:sameIndividualAs deprecated
@@ -98,15 +97,15 @@ class OwlTestSuite(unittest.TestCase):
         store.open(RDFLIB_CONNECTION)
         self.ruleStore=N3RuleStore()
         self.ruleGraph = Graph(self.ruleStore)
-        self.ruleFactsGraph = Graph(store)            
-        for fileN in ['pD-rules.n3']:
-            self.ruleGraph.parse(open(fileN),format='n3')        
-            self.ruleFactsGraph.parse(open(fileN),format='n3')
+        self.ruleFactsGraph = Graph(store)
+        self.ruleGraph.parse(StringIO(non_DHL_OWL_Semantics),format='n3')
         self.network = ReteNetwork(self.ruleStore)
-        renderNetwork(self.network,nsMap=nsMap).write_graphviz('owl-rules.dot')
+        
+        #renderNetwork(self.network,nsMap=nsMap).write_graphviz('owl-rules.dot')
     def tearDown(self):
         pass
     
+    
     def testOwl(self): 
         testData = {}       
         for manifest in glob('OWL/*/Manifest*.rdf'):
@@ -139,14 +138,16 @@ class OwlTestSuite(unittest.TestCase):
                 if status == 'APPROVED':
                     assert os.path.exists('.'.join([premiseFile,'rdf'])) 
                     assert os.path.exists('.'.join([conclusionFile,'rdf']))
+                    print "<%s> :- <%s>"%('.'.join([conclusionFile,'rdf']),
+                                          '.'.join([premiseFile,'rdf']))
                     store = plugin.get(RDFLIB_STORE,Store)()
                     store.open(RDFLIB_CONNECTION)
                     factGraph = Graph(store)
                     factGraph.parse(open('.'.join([premiseFile,'rdf'])))
                     allFacts = ReadOnlyGraphAggregate([factGraph,self.ruleFactsGraph])
+                    MapDLPtoNetwork(self.network,factGraph)
                     print self.network                                      
                     start = time.time()  
-                    self.network.feedFactsToAdd(generateTokenSet(self.ruleFactsGraph))                    
                     self.network.feedFactsToAdd(generateTokenSet(factGraph))                    
                     sTime = time.time() - start
                     if sTime > 1:
@@ -156,7 +157,7 @@ class OwlTestSuite(unittest.TestCase):
                         sTimeStr = "%s milli seconds"%sTime
                     print "Time to calculate closure on working memory: ",sTimeStr
                     print self.network
-                    tNodeOrder = [tNode for tNode in self.network.terminalNodes if self.network.instanciations[tNode]]
+                    tNodeOrder = [tNode for tNode in self.network.terminalNodes if tNode in self.network.instanciations]
                     tNodeOrder.sort(key=lambda x:self.network.instanciations[x],reverse=True)
                     for termNode in []:#tNodeOrder:
                         lhsF,rhsF = termNode.ruleFormulae
@@ -169,15 +170,15 @@ class OwlTestSuite(unittest.TestCase):
                     for triple in expectedFacts.parse('.'.join([conclusionFile,'rdf'])):
                         closureGraph = ReadOnlyGraphAggregate([self.network.inferredFacts,factGraph])
                         if triple not in self.network.inferredFacts and triple not in factGraph:
-                            print "missing triple %s"%(repr(triple))
+                            print "missing triple %s"%(pformat(triple))
                             print manifest
                             print "feature: ", feature
                             print description
-                            #pprint(list(self.network.inferredFacts))                            
-                            return
-#                        else:
-#                            print "=== Passed! ==="
-#                            pprint(list(self.network.inferredFacts))
+                            pprint(list(self.network.inferredFacts))                            
+                            raise Exception ("Failed test: "+feature)
+                        else:
+                            print "=== Passed! ==="
+                            #pprint(list(self.network.inferredFacts))
                     print "\n"
                     testData[manifest] = sTimeStr
                     store.rollback()
